@@ -158,7 +158,7 @@ args_craigmr = (:A, :b)
 kwargs_craigmr = (:M, :N, :ldiv, :sqd, :λ, :atol, :rtol, :itmax, :timemax, :verbose, :history, :callback, :iostream)
 
 @eval begin
-  function craigmr!(workspace :: CraigmrWorkspace{T,FC,S}, $(def_args_craigmr...); $(def_kwargs_craigmr...)) where {T <: AbstractFloat, FC <: FloatOrComplex{T}, S <: AbstractVector{FC}}
+  function craigmr!(workspace :: CraigmrWorkspace{T,FC,Sm,Sn}, $(def_args_craigmr...); $(def_kwargs_craigmr...)) where {T <: AbstractFloat, FC <: FloatOrComplex{T}, Sm <: AbstractVector{FC}, Sn <: AbstractVector{FC}}
 
     # Timer
     start_time = time_ns()
@@ -179,15 +179,15 @@ kwargs_craigmr = (:M, :N, :ldiv, :sqd, :λ, :atol, :rtol, :itmax, :timemax, :ver
 
     # Check type consistency
     eltype(A) == FC || @warn "eltype(A) ≠ $FC. This could lead to errors or additional allocations in operator-vector products."
-    ktypeof(b) == S || error("ktypeof(b) must be equal to $S")
+    ktypeof(b) == Sm || error("ktypeof(b) must be equal to $Sm")
 
     # Compute the adjoint of A
     Aᴴ = A'
 
     # Set up workspace.
-    allocate_if(!MisI, workspace, :u, S, workspace.y)  # The length of u is m
-    allocate_if(!NisI, workspace, :v, S, workspace.x)  # The length of v is n
-    allocate_if(λ > 0, workspace, :q, S, workspace.x)  # The length of q is n
+    allocate_if(!MisI, workspace, :u, Sm, workspace.y)  # The length of u is m
+    allocate_if(!NisI, workspace, :v, Sn, workspace.x)  # The length of v is n
+    allocate_if(λ > 0, workspace, :q, Sn, workspace.x)  # The length of q is n
     x, Nv, Aᴴu, d, y, Mu = workspace.x, workspace.Nv, workspace.Aᴴu, workspace.d, workspace.y, workspace.Mu
     w, wbar, Av, q, stats = workspace.w, workspace.wbar, workspace.Av, workspace.q, workspace.stats
     rNorms, ArNorms = stats.residuals, stats.Aresiduals
